;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x14182e9b19b27fe1a20c4e9634010733 . "./code.cg2js/cg-acache.fasl")
(#x5f7ec80b4113378d43c36e92f238c497 . "./code.cg2js/cg-base.fasl")
(#xc6ad3411b273601b7a2b1723e53a2572 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x6821a3d0ef243526ba85a5c34f1ddef3 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xdd2d266a47de383643dc7a2d62f1c670 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x23ad2c41fee8f058956329b3152ee564 . "./code.cg2js/cg-button.fasl")
(#x84357296ce2392a3ae7409ce09f30457 . "./code.cg2js/cg-calendar.fasl")
(#xbcb339bcfb1ec6229e0e3bf8f31a7e86 . "./code.cg2js/cg-caret.fasl")
(#xa53817d3275e5d40067774684b1421af . "./code.cg2js/cg-chart-or-plot.fasl")
(#xfdd8f3678a3cc346532568d1f31bef65 . "./code.cg2js/cg-chart-widget.fasl")
(#x7c22842bc4755e00992b6c6eacb27d24 . "./code.cg2js/cg-check-box.fasl")
(#xbc9ae4d67e221428be97e2be446a5745 . "./code.cg2js/cg-choice-list.fasl")
(#xb7849921f13fd6f685fd97831b833192 . "./code.cg2js/cg-choose-printer.fasl")
(#x92a482818449a6fba80a7bdb6241d1d4 . "./code.cg2js/cg-class-grid.fasl")
(#x515776932cb675c3902fb2f4049d4405 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xb09c154d407c1bb6e4355e6b3686245b . "./code.cg2js/cg-class-support.fasl")
(#x1e26b198052f3cc89b1a5f561ad20149 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x73e7aa52a5df3460981547a32cc34852 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x0993a0d3dbbd674610afaa78ac5127b7 . "./code.cg2js/cg-clipboard.fasl")
(#xcbf191ecbe85216a3de12be63d74003d . "./code.cg2js/cg-color-dialog.fasl")
(#x6164f0d4ca306cc33268bf3584f75b2d . "./code.cg2js/cg-combo-box.fasl")
(#x416ebdb1e210a2018d790212666feef5 . "./code.cg2js/cg-common-control.fasl")
(#x146c66d8266a2d691f494f2383712ca1 . "./code.cg2js/cg-comtab.fasl")
(#xc15648a2054fbbac46c7471e27987135 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xc26e5e8f02852ce04ce8c1da11316c59 . "./code.cg2js/cg-curve.fasl")
(#x1317001b6521f05af84b358d75be429d . "./code.cg2js/cg-dde-utils.fasl")
(#xb2c63c4f7f4fb982aaf3bd6acef555a4 . "./code.cg2js/cg-dev.fasl")
(#x7c9e4f66052be7d436a93b597e272744 . "./code.cg2js/cg-dialog-item.fasl")
(#xc4dd0fb1d0a6172b413b6919a959de3c . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x509910d2ee48253c7dc3c5db25dea557 . "./code.cg2js/cg-directory-dialog.fasl")
(#xb6caca5c73ba5c5d36c3a679be8d1d0f . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x79e01b1b38bdfe8192c41408b05df202 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xdf5c51077b333d53b14ec3ec4b43667c . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xe3c6576e1d0e92dd3b33f2ece26cc244 . "./code.cg2js/cg-drawable.fasl")
(#x14a171432410b0049e36dd46305caec7 . "./code.cg2js/cg-dropping-outline.fasl")
(#x6cc2c2aa9f0a997fc9c76ec2c86e118a . "./code.cg2js/cg-edit-in-place.fasl")
(#x9b036e49aa4e1dcee117db96ee39f34d . "./code.cg2js/cg-editable-text.fasl")
(#x36de1b8c1be3e61d49da6199d039dc49 . "./code.cg2js/cg-file-dialog.fasl")
(#x74cefdbf34bf1ad61a201a84fd7545e3 . "./code.cg2js/cg-fill-texture.fasl")
(#xc546c64fc253d90bba77021ee19fb7f5 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x1d8cc38fef0f2d3e668328123e942feb . "./code.cg2js/cg-font-dialog.fasl")
(#x021e37029980dffec0118a4b42865e93 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xda228e3633151ea95351335c185ac573 . "./code.cg2js/cg-get-pixmap.fasl")
(#x5e336b6bdaf0c3459445f29850bcb0d8 . "./code.cg2js/cg-get-position.fasl")
(#xe0e27409a2a6d3d23eecb1ae294fd880 . "./code.cg2js/cg-graphics-context.fasl")
(#xdd71933e8c949ef96cc6c62cc0ec30f6 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x8568e9ae49f655d36c4aebaa98c8a9ea . "./code.cg2js/cg-grid-widget.fasl")
(#x66c7242be4ffab42abe970fcb93b6c32 . "./code.cg2js/cg-group-box.fasl")
(#x85171ed68c43d536afaccbee1b8bd61b . "./code.cg2js/cg-header-control.fasl")
(#x0020d9b84e4f242b6628cd47704f887b . "./code.cg2js/cg-hotspot.fasl")
(#x8904eebabaea43d65c15fd3b297e4125 . "./code.cg2js/cg-html-dialog.fasl")
(#x24a0feb9a8ce7f2d91a1ac31e191ea0e . "./code.cg2js/cg-html-widget.fasl")
(#x14dccfc3b2d12b8b25a5b6a61cce33e2 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x7217d0ea2bf289b700395fae8039d707 . "./code.cg2js/cg-icon.fasl")
(#x2c9d8301f462d96acb68f3c66c37f2ec . "./code.cg2js/cg-ie.fasl")
(#xf144cdbe8dfd4e85ec30e0617e8a7171 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x14c66c541be95bbf84fdbb4eeb179c0f . "./code.cg2js/cg-intersect.fasl")
(#x444433f4f618755dc59c18ff01e48ed6 . "./code.cg2js/cg-item-list.fasl")
(#x7fc38183a5f5875dc3144f85e2b4d8e2 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xa5c6ce1899903b4b53545aeae5725624 . "./code.cg2js/cg-lamp.fasl")
(#x289db680f474fb473893032ffc281e6c . "./code.cg2js/cg-layout.fasl")
(#x0c36d5c024dd33fd88817400727c0d80 . "./code.cg2js/cg-lettered-menu.fasl")
(#x1f20f6f892fc8308f9cd34e38052b2cf . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xfca1a6a4b44422403fb70045486650c1 . "./code.cg2js/cg-lisp-text.fasl")
(#xec770a060fc7036caf0a8827d461eed3 . "./code.cg2js/cg-lisp-widget.fasl")
(#x581ecb8c84bf7d5c1d805a6f55d77a56 . "./code.cg2js/cg-list-view.fasl")
(#x5c68ac06b016d913e6bb5c6af56b5962 . "./code.cg2js/cg-mci.fasl")
(#xbe09247930af396ba0d27354ae87bd21 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x51e9dea0aaedfdd212a7fdbf01541d7d . "./code.cg2js/cg-menu.fasl")
(#x76edb0b2a8360eade84e14c558af59f4 . "./code.cg2js/cg-message-dialog.fasl")
(#x95c310854649649adc2970858052a5a4 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf80d0027b224b9ceb8cc1b6c95977539 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x73eebf543819edd985b24137df14e153 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x05f453d1fda8a467e3bf64f7bb5e53fe . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x4b392f9327b8d0f353759dd2d3663fc1 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x2b19c34814cd6c14c8a6ed47d8a9824a . "./code.cg2js/cg-nodes.fasl")
(#x10aaa01f28a07a0f575a19e84fe515cc . "./code.cg2js/cg-object-editor-layout.fasl")
(#x8298f1df822464fd45175c96544c2c2c . "./code.cg2js/cg-object-editor.fasl")
(#xfd4e7e1eb9aceba3c51e304a9435ce5d . "./code.cg2js/cg-ocx.fasl")
(#x27e96d6b88e6a5c3b18074322415ea3c . "./code.cg2js/cg-os-widget.fasl")
(#x4590cab16e2c270bf46bd641e231768c . "./code.cg2js/cg-os-window.fasl")
(#xbfee51fc82617a4dfd7bfdaa7d5a6b4d . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x5ece2ab3337283e49c8e13e4967705d6 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xb52b45d63f80a02a3a0541aaaba0bff0 . "./code.cg2js/cg-outline.fasl")
(#x146026347f70ba995a9749c1dfe3ad06 . "./code.cg2js/cg-palette.fasl")
(#x0cebd77beaef4d5cb77a3d4dfdb02c80 . "./code.cg2js/cg-paren-matching.fasl")
(#x541d7b887f35fd905f5bbe4d50798a53 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc29497a2265e61b7234592a434138024 . "./code.cg2js/cg-picture-widget.fasl")
(#x942c6f60913c90c9582b1756548f0011 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x2e098bae4886452e3dda4d40cc631cbb . "./code.cg2js/cg-pixmap-printing.fasl")
(#xe2be2f5dd0fa480e0de1f09378e10cb5 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x3f3546100998e24aab65c9e63d1366f6 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x388210a40e3b2e5d7d399072a06c5993 . "./code.cg2js/cg-pixmap.fasl")
(#x2dabd9bc24c182694aa6f1b528e60c83 . "./code.cg2js/cg-plot-widget.fasl")
(#x4013f2601e3998debb3fd82d8c402130 . "./code.cg2js/cg-printing.fasl")
(#x3c48d3c82a4dab24907d29898c070f3b . "./code.cg2js/cg-progress-indicator.fasl")
(#x7c9446e26cf99986e60cc501c2e5af0b . "./code.cg2js/cg-project-window.fasl")
(#xf980e2d07fb7404fd2fbd91100f1b2ab . "./code.cg2js/cg-property.fasl")
(#xbc78fbd2d06497f4e667f4eabc684e33 . "./code.cg2js/cg-radio-button.fasl")
(#x6d3c2c5f5b9e69ce404d26fce1f07952 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x2bd97c94bbeac1624c9da203c453f4ec . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xa3823e58ee48f932dec4fdba3d3196ff . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xff24db0fba36c141406a1b14cf64f270 . "./code.cg2js/cg-rich-edit.fasl")
(#xc1b8333666691b58cee8656a3f03ff13 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x19c40fc5656698f4d52ecd4e8906be53 . "./code.cg2js/cg-scaling-stream.fasl")
(#x8657838be23824e27595ec78b619ca71 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x0b745a4b8e4542b79579e8a2edfa8f1c . "./code.cg2js/cg-scroll-bar.fasl")
(#x9936215ccfaf1eac581157948287d304 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x708665a2d7d69b4e37165fa8be6af8d6 . "./code.cg2js/cg-selected-object.fasl")
(#x8b5d172e2d0eacf747f9ba52e59232a0 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xd8b2aeb0e93381ceedbfd78eae7ba5b4 . "./code.cg2js/cg-split-bar.fasl")
(#x69358346af4abfd4a33fb50e52543c57 . "./code.cg2js/cg-static-text.fasl")
(#x14dd1a958a8516b9f25b25cfa646ba5c . "./code.cg2js/cg-status-bar.fasl")
(#xfa3bba8253c7b3d9d9e2bc8384dbcd73 . "./code.cg2js/cg-string-dialog.fasl")
(#xf2e138a228323bffbd09ecc91b51e984 . "./code.cg2js/cg-tab-control.fasl")
(#xec7dc430c1f5d55a6b7aadcc062e82c8 . "./code.cg2js/cg-template-string.fasl")
(#xc06f336321786d7535f4a6bcac5d5620 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x717ce1f09bea94f7657e49a6c1d4e47a . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x0d1036634fc35325dcdbedccdd74b5f6 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x2d4550a57fb69d9412f75469c3d356ea . "./code.cg2js/cg-text-or-combo.fasl")
(#xab558bcfc2fb4057daba52f48447e8fb . "./code.cg2js/cg-text-widget.fasl")
(#x09ef8b9d31fb0386ef75dc5163012f9e . "./code.cg2js/cg-timer.fasl")
(#xab98f180bf570f92e4b88290f6805826 . "./code.cg2js/cg-toggling-widget.fasl")
(#x3df8394856608015707aaa03ec0a5a5f . "./code.cg2js/cg-toolbar.fasl")
(#xdd00777e7ba528ac0a1fc0278f7220f9 . "./code.cg2js/cg-tooltip.fasl")
(#x745459de9976421be4600d6deb6ad4da . "./code.cg2js/cg-touch.fasl")
(#x3a9963bb8434922590ec742b81b2098c . "./code.cg2js/cg-trackbar.fasl")
(#x50574c2e8ab6cd5c00aae3d31c6f81b4 . "./code.cg2js/cg-tray.fasl")
(#x362fbb3d30f3875fc8b09935ff31376d . "./code.cg2js/cg-up-down-control.fasl")
(#x7623652932799208e32dfadc459c14c1 . "./code.cg2js/cg-utility-dialog.fasl")
(#x7d31343b73014f356d0caba112a2944d . "./code.cg2js/cg-web-browser-dde.fasl")
(#x5334b7ee674cc92e1f162f7b76a56978 . "./code.cg2js/cg-web-browser.fasl")
(#xcd2e0d6c0bd235d506f2e4ba8168acae . "./code.cg2js/cg-wrap-string.fasl")
(#x3d0ab4b5d066f4f5abbe3b376ed62f96 . "./code.cg2js/cg-yes-no-list.fasl")
(#x01e8f93a29c8fdcae3fe67d3202f06aa . "./code.cg2js/cg-yes-no-string.fasl")
(#x34c92d10b41338e679ca2fdd51710ed5 . "./code.cg2js/cg.fasl")
(#x9d8073925e78cbecd114b7ff069553d9 . "./code.idejs/ide-acache.fasl")
(#xe3d9dc1348290c347f56d8c4513f8503 . "./code.idejs/ide-apropos.fasl")
(#xc8f19876573924d6e8eb55a826560ddd . "./code.idejs/ide-base.fasl")
(#x19f6c6920d54e055cd50febfd2bf4787 . "./code.idejs/ide-class-browser.fasl")
(#x30e2333f93b512826b3efa41a6303354 . "./code.idejs/ide-class-grapher.fasl")
(#x9d9fa4215e0151db1b5c286818675687 . "./code.idejs/ide-clipboard.fasl")
(#x2e35e2e588b7e16308372d9217086749 . "./code.idejs/ide-dev.fasl")
(#xb7184ebd68eff4d0e2ad3580664ffbf2 . "./code.idejs/ide-editor.fasl")
(#xdc38da381707f5743d708733650b1f54 . "./code.idejs/ide-find-definitions.fasl")
(#xc316bcb55098025f23b5b5af2e8ec2d7 . "./code.idejs/ide-find-in-files.fasl")
(#x2b5bcc94b6edb85ab3df425ecafb1759 . "./code.idejs/ide-form.fasl")
(#x89bda9e2dc07eea001f7b4333a0636c6 . "./code.idejs/ide-grapher.fasl")
(#x1fa7af8940facd28db9db8abd6b8aec6 . "./code.idejs/ide-inspector.fasl")
(#xf5758ba662d5f029996052f60a137d93 . "./code.idejs/ide-intro.fasl")
(#xe5e46445f2bb2c919a0ca231121f9d35 . "./code.idejs/ide-keyshort.fasl")
(#xf5fc4e8b1bab3c93763a066c24cb698a . "./code.idejs/ide-navigator.fasl")
(#x2ae442a3b377521e7e382c7dcab77733 . "./code.idejs/ide-package-list.fasl")
(#xea009039cb7abb393d6cddb905983793 . "./code.idejs/ide-patch-dialog.fasl")
(#xc08d7cd117b1c0436a700bc82bbdc381 . "./code.idejs/ide-process-browser.fasl")
(#xf508a3e836f4bd3e59fbdb82669170d1 . "./code.idejs/ide-project.fasl")
(#x8c42007c57e88c6d25f4323c93b54888 . "./code.idejs/ide-trace.fasl")
(#x8ef773d21d40e151ca9bfac3f1b52257 . "./code.idejs/ide-window-list.fasl")
(#x55676c584f2dffb695a93a62d4063271 . "./code.idejs/ide.fasl")
(#x7f480facc219636edfe865d5e108e9cb . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
